@@ -1,3 +1,4 @@
+
 /*
  *
  *    OPEN-XCHANGE legal information
@@ -50,29 +51,37 @@
 package com.openexchange.google.api.client;
 
 import java.security.GeneralSecurityException;
-import java.util.HashMap;
-import java.util.Map;
+import javax.net.ssl.SSLHandshakeException;
+import org.json.JSONException;
+import org.json.JSONObject;
 import org.scribe.builder.ServiceBuilder;
 import org.scribe.builder.api.Google2Api;
+import org.scribe.exceptions.OAuthException;
 import org.scribe.model.Token;
+import org.slf4j.Logger;
 import com.google.api.client.googleapis.auth.oauth2.GoogleCredential;
 import com.google.api.client.http.javanet.NetHttpTransport;
 import com.google.api.client.json.JsonFactory;
 import com.google.api.client.json.jackson2.JacksonFactory;
+import com.openexchange.cluster.lock.ClusterLockService;
+import com.openexchange.cluster.lock.ClusterTask;
+import com.openexchange.cluster.lock.policies.ExponentialBackOffRetryPolicy;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.cascade.ConfigView;
+import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.exception.ExceptionUtils;
 import com.openexchange.exception.OXException;
 import com.openexchange.google.api.client.services.Services;
 import com.openexchange.java.Strings;
-import com.openexchange.oauth.API;
-import com.openexchange.oauth.DefaultOAuthToken;
+import com.openexchange.net.ssl.exception.SSLExceptionCode;
+import com.openexchange.oauth.KnownApi;
+import com.openexchange.oauth.AbstractReauthorizeClusterTask;
 import com.openexchange.oauth.OAuthAccount;
-import com.openexchange.oauth.OAuthConstants;
 import com.openexchange.oauth.OAuthExceptionCodes;
 import com.openexchange.oauth.OAuthService;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
 
-
 /**
  * {@link GoogleApiClients} - Utility class for Google API client.
  *
@@ -81,6 +90,9 @@ import com.openexchange.session.Session;
  */
 public class GoogleApiClients {
 
+    /** The refresh threshold in seconds: <code>60</code> */
+    public static final int REFRESH_THRESHOLD = 60;
+
     /**
      * Initializes a new {@link GoogleApiClients}.
      */
@@ -121,7 +133,7 @@ public class GoogleApiClients {
         }
 
         // Get default Google account
-        OAuthAccount defaultAccount = oAuthService.getDefaultAccount(API.GOOGLE, session);
+        OAuthAccount defaultAccount = oAuthService.getDefaultAccount(KnownApi.GOOGLE, session);
 
         if (reacquireIfExpired) {
             try {
@@ -132,30 +144,13 @@ public class GoogleApiClients {
 
                 // Check expiry
                 int expiry = scribeOAuthService.getExpiry(defaultAccount.getToken());
-                if (expiry < 300) {
-                    // Less than 5 minutes to live -> refresh token!
-                    String refreshToken = defaultAccount.getSecret();
-                    Token accessToken = scribeOAuthService.getAccessToken(new Token(defaultAccount.getToken(), defaultAccount.getSecret()), null);
-                    if (!Strings.isEmpty(accessToken.getSecret())) {
-                        refreshToken = accessToken.getSecret();
-                    }
-                    // Update account
-                    int accountId = defaultAccount.getId();
-                    Map<String, Object> arguments = new HashMap<String, Object>(3);
-                    arguments.put(OAuthConstants.ARGUMENT_REQUEST_TOKEN, new DefaultOAuthToken(accessToken.getToken(), refreshToken));
-                    arguments.put(OAuthConstants.ARGUMENT_SESSION, session);
-                    oAuthService.updateAccount(accountId, arguments, session.getUserId(), session.getContextId());
-
-                    // Reload
-                    defaultAccount = oAuthService.getAccount(accountId, session, session.getUserId(), session.getContextId());
+                if (expiry < REFRESH_THRESHOLD) {
+                    // Less than 1 minute to live -> refresh token!
+                    ClusterLockService clusterLockService = Services.getService(ClusterLockService.class);
+                    defaultAccount = clusterLockService.runClusterTask(new GoogleReauthorizeClusterTask(session, defaultAccount), new ExponentialBackOffRetryPolicy());
                 }
             } catch (org.scribe.exceptions.OAuthException e) {
-                // Failed to request new access token
-                if (e.getMessage().indexOf("\"invalid_grant\"") >= 0) {
-                    // Refresh token in use is invalid/expired
-                    throw OAuthExceptionCodes.INVALID_ACCOUNT_EXTENDED.create(e, defaultAccount.getDisplayName(), defaultAccount.getId());
-                }
-                throw OAuthExceptionCodes.OAUTH_ERROR.create(e, e.getMessage());
+                throw handleScribeOAuthException(e, defaultAccount);
             }
         }
 
@@ -197,28 +192,21 @@ public class GoogleApiClients {
         OAuthAccount googleAccount = oAuthService.getAccount(accountId, session, session.getUserId(), session.getContextId());
 
         if (reacquireIfExpired) {
-            // Create Scribe Google OAuth service
-            final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
-            serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
-            Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
+            try {
+                // Create Scribe Google OAuth service
+                final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
+                serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
+                Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
 
-            // Check expiry
-            int expiry = scribeOAuthService.getExpiry(googleAccount.getToken());
-            if (expiry < 300) {
-                // Less than 5 minutes to live -> refresh token!
-                String refreshToken = googleAccount.getSecret();
-                Token accessToken = scribeOAuthService.getAccessToken(new Token(googleAccount.getToken(), googleAccount.getSecret()), null);
-                if (!Strings.isEmpty(accessToken.getSecret())) {
-                    refreshToken = accessToken.getSecret();
+                // Check expiry
+                int expiry = scribeOAuthService.getExpiry(googleAccount.getToken());
+                if (expiry < REFRESH_THRESHOLD) {
+                    // Less than 1 minute to live -> refresh token!
+                    ClusterLockService clusterLockService = Services.getService(ClusterLockService.class);
+                    googleAccount = clusterLockService.runClusterTask(new GoogleReauthorizeClusterTask(session, googleAccount), new ExponentialBackOffRetryPolicy());
                 }
-                // Update account
-                Map<String, Object> arguments = new HashMap<String, Object>(3);
-                arguments.put(OAuthConstants.ARGUMENT_REQUEST_TOKEN, new DefaultOAuthToken(accessToken.getToken(), refreshToken));
-                arguments.put(OAuthConstants.ARGUMENT_SESSION, session);
-                oAuthService.updateAccount(accountId, arguments, session.getUserId(), session.getContextId());
-
-                // Reload
-                googleAccount = oAuthService.getAccount(accountId, session, session.getUserId(), session.getContextId());
+            } catch (org.scribe.exceptions.OAuthException e) {
+                throw handleScribeOAuthException(e, googleAccount);
             }
         }
 
@@ -238,14 +226,17 @@ public class GoogleApiClients {
             return -1L;
         }
 
-        // Create Scribe Google OAuth service
-        final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
-        serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
-        Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
+        try {
+            // Create Scribe Google OAuth service
+            final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
+            serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
+            Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
 
-        // Check expiry
-        int expiry = scribeOAuthService.getExpiry(googleAccount.getToken());
-        return expiry;
+            // Check expiry
+            return scribeOAuthService.getExpiry(googleAccount.getToken());
+        } catch (org.scribe.exceptions.OAuthException e) {
+            throw handleScribeOAuthException(e, googleAccount);
+        }
     }
 
     /**
@@ -256,7 +247,7 @@ public class GoogleApiClients {
      * @return The non-expired candidate or <code>null</code> if given account appears to have enough time left
      * @throws OXException If a non-expired candidate cannot be returned
      */
-    public static OAuthAccount ensureNonExpiredGoogleAccount(OAuthAccount googleAccount, Session session) throws OXException {
+    public static OAuthAccount ensureNonExpiredGoogleAccount(final OAuthAccount googleAccount, final Session session) throws OXException {
         if (null == googleAccount) {
             return googleAccount;
         }
@@ -267,37 +258,105 @@ public class GoogleApiClients {
             throw ServiceExceptionCode.absentService(OAuthService.class);
         }
 
-        // Create Scribe Google OAuth service
-        final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
-        serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
-        Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
+        try {
+            // Create Scribe Google OAuth service
+            final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
+            serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
+            final Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
 
-        // Check expiry
-        int expiry = scribeOAuthService.getExpiry(googleAccount.getToken());
-        if (expiry >= 300) {
-            // More than 5 minutes to live
-            return null;
+            // Check expiry
+            int expiry = scribeOAuthService.getExpiry(googleAccount.getToken());
+            if (expiry >= REFRESH_THRESHOLD) {
+                // More than 1 minute to live
+                return null;
+            }
+
+            ClusterLockService clusterLockService = Services.getService(ClusterLockService.class);
+            return clusterLockService.runClusterTask(new GoogleReauthorizeClusterTask(session, googleAccount), new ExponentialBackOffRetryPolicy());
+        } catch (org.scribe.exceptions.OAuthException e) {
+            throw handleScribeOAuthException(e, googleAccount);
         }
-
-        // Less than 5 minutes to live -> refresh token!
-        String refreshToken = googleAccount.getSecret();
-        Token accessToken = scribeOAuthService.getAccessToken(new Token(googleAccount.getToken(), googleAccount.getSecret()), null);
-        if (!Strings.isEmpty(accessToken.getSecret())) {
-            refreshToken = accessToken.getSecret();
-        }
-        // Update account
-        int accountId = googleAccount.getId();
-        Map<String, Object> arguments = new HashMap<String, Object>(3);
-        arguments.put(OAuthConstants.ARGUMENT_REQUEST_TOKEN, new DefaultOAuthToken(accessToken.getToken(), refreshToken));
-        arguments.put(OAuthConstants.ARGUMENT_SESSION, session);
-        oAuthService.updateAccount(accountId, arguments, session.getUserId(), session.getContextId());
-
-        // Reload
-        return oAuthService.getAccount(accountId, session, session.getUserId(), session.getContextId());
     }
 
     /**
-     * Gets the Google credentials from default OAuth account.
+     * Gets the expiry (in seconds) for given Google OAuth account
+     *
+     * @param googleAccount The Google OAuth account to check
+     * @param session The associated session
+     * @return The expiry in seconds or <code>-1</code> if access token is already expired
+     * @throws OXException If expiry cannot be returned
+     * @throws IllegalArgumentException If provided account is <code>null</code>
+     */
+    public static int getExpiryForGoogleAccount(final OAuthAccount googleAccount, final Session session) throws OXException {
+        if (null == googleAccount) {
+            throw new IllegalArgumentException("Account must not be null");
+        }
+
+        // Get OAuth service
+        final OAuthService oAuthService = Services.optService(OAuthService.class);
+        if (null == oAuthService) {
+            throw ServiceExceptionCode.absentService(OAuthService.class);
+        }
+
+        try {
+            // Create Scribe Google OAuth service
+            final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
+            serviceBuilder.apiKey(googleAccount.getMetaData().getAPIKey(session)).apiSecret(googleAccount.getMetaData().getAPISecret(session));
+            final Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
+
+            // Check expiry
+            return scribeOAuthService.getExpiry(googleAccount.getToken());
+        } catch (org.scribe.exceptions.OAuthException e) {
+            throw handleScribeOAuthException(e, googleAccount);
+        }
+    }
+
+    static OXException handleScribeOAuthException(OAuthException e, OAuthAccount googleAccount) {
+        if (ExceptionUtils.isEitherOf(e, SSLHandshakeException.class)) {
+            return SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e, "www.googleapis.com");
+        }
+
+        String exMessage = e.getMessage();
+        String errorMsg = parseErrorFrom(exMessage);
+        if (Strings.isEmpty(errorMsg)) {
+            return OAuthExceptionCodes.OAUTH_ERROR.create(e, exMessage);
+        }
+        if (exMessage.contains("invalid_grant")) {
+            if (null != googleAccount) {
+                return OAuthExceptionCodes.INVALID_ACCOUNT_EXTENDED.create(e, googleAccount.getDisplayName(), googleAccount.getId());
+            }
+            return OAuthExceptionCodes.INVALID_ACCOUNT.create(e, new Object[0]);
+        }
+        return OAuthExceptionCodes.OAUTH_ERROR.create(exMessage, e);
+    }
+
+    private static String parseErrorFrom(String message) {
+        if (Strings.isEmpty(message)) {
+            return null;
+        }
+
+        String marker = "Can't extract a token from this: '";
+        int pos = message.indexOf(marker);
+        if (pos < 0) {
+            return null;
+        }
+
+        try {
+            JSONObject jo = new JSONObject(message.substring(pos + marker.length(), message.length() - 1));
+            return jo.optString("error", null);
+        } catch (JSONException e) {
+            // Apparent no JSON response
+            return null;
+        }
+    }
+
+    /**
+     * Gets the Google credentials from <b>default</b> OAuth account.
+     * <p>
+     * <div style="margin-left: 0.1in; margin-right: 0.5in; margin-bottom: 0.1in; background-color:#FFDDDD;">
+     * Please use {@link #getCredentials(OAuthAccount, Session)} if a concrete Google OAuth account is supposed to be used
+     * </div>
+     * <p>
      *
      * @param session The associated session
      * @return The Google credentials from default OAuth account
@@ -313,19 +372,22 @@ public class GoogleApiClients {
      *
      * @param googleOAuthAccount The Google OAuth account
      * @param session The associated session
-     * @return The Google credentials from given OAuth account
+     * @return The Google credentials from given OAuth account or <code>null</code> if either of arguments is <code>null</code>
      * @throws OXException If Google credentials cannot be returned
      */
     public static GoogleCredential getCredentials(OAuthAccount googleOAuthAccount, Session session) throws OXException {
+        if (null == googleOAuthAccount) {
+            return null;
+        }
+        if (null == session) {
+            return null;
+        }
         try {
             // Initialize transport
             NetHttpTransport transport = new NetHttpTransport.Builder().doNotValidateCertificate().build();
 
             // Build credentials
-            return new GoogleCredential.Builder()
-            .setClientSecrets(googleOAuthAccount.getMetaData().getAPIKey(session), googleOAuthAccount.getMetaData().getAPISecret(session))
-            .setJsonFactory(JSON_FACTORY).setTransport(transport).build()
-            .setRefreshToken(googleOAuthAccount.getSecret()).setAccessToken(googleOAuthAccount.getToken());
+            return new GoogleCredential.Builder().setClientSecrets(googleOAuthAccount.getMetaData().getAPIKey(session), googleOAuthAccount.getMetaData().getAPISecret(session)).setJsonFactory(JSON_FACTORY).setTransport(transport).build().setRefreshToken(googleOAuthAccount.getSecret()).setAccessToken(googleOAuthAccount.getToken());
         } catch (GeneralSecurityException e) {
             throw OAuthExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
@@ -336,9 +398,56 @@ public class GoogleApiClients {
      *
      * @return The product name
      */
-    public static String getGoogleProductName() {
+    public static String getGoogleProductName(Session session) {
+        if (null != session) {
+            ConfigViewFactory factory = Services.getService(ConfigViewFactory.class);
+            if (null != factory) {
+                try {
+                    ConfigView view = factory.getView(session.getUserId(), session.getContextId());
+                    return view.opt("com.openexchange.oauth.google.productName", String.class, "");
+                } catch (OXException e) {
+                    Logger logger = org.slf4j.LoggerFactory.getLogger(GoogleApiClients.class);
+                    logger.warn("Failed to look-up property \"com.openexchange.oauth.google.productName\" using config-cascade. Falling back to configuration service...", e);
+                }
+            }
+        }
+
         ConfigurationService configService = Services.getService(ConfigurationService.class);
         return null == configService ? "" : configService.getProperty("com.openexchange.oauth.google.productName", "");
     }
 
+    //////////////////////// HELPERS /////////////////////////////
+
+    /**
+     * {@link GoogleReauthorizeClusterTask}
+     */
+    private static class GoogleReauthorizeClusterTask extends AbstractReauthorizeClusterTask implements ClusterTask<OAuthAccount> {
+
+        /**
+         * Initialises a new {@link GoogleApiClients.GoogleReauthorizeClusterTask}.
+         */
+        public GoogleReauthorizeClusterTask(Session session, OAuthAccount cachedAccount) {
+            super(Services.getServiceLookup(), session, cachedAccount);
+        }
+
+        /*
+         * (non-Javadoc)
+         *
+         * @see com.openexchange.cluster.lock.ClusterTask#perform()
+         */
+        @Override
+        public Token reauthorize() throws OXException {
+            final ServiceBuilder serviceBuilder = new ServiceBuilder().provider(Google2Api.class);
+            serviceBuilder.apiKey(getCachedAccount().getMetaData().getAPIKey(getSession())).apiSecret(getCachedAccount().getMetaData().getAPISecret(getSession()));
+            Google2Api.GoogleOAuth2Service scribeOAuthService = (Google2Api.GoogleOAuth2Service) serviceBuilder.build();
+
+            // Refresh the token
+            try {
+                return scribeOAuthService.getAccessToken(new Token(getCachedAccount().getToken(), getCachedAccount().getSecret()), null);
+            } catch (OAuthException e) {
+                OAuthAccount dbAccount = getDBAccount();
+                throw handleScribeOAuthException(e, dbAccount);
+            }
+        }
+    }
 }
