@@ -54,17 +54,21 @@ import java.util.Hashtable;
 import com.openexchange.ajax.requesthandler.osgiservice.AJAXModuleActivator;
 import com.openexchange.capabilities.CapabilityChecker;
 import com.openexchange.capabilities.CapabilityService;
+import com.openexchange.cluster.lock.ClusterLockService;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.notify.hostname.HostnameService;
-import com.openexchange.oauth.OAuthHTTPClientFactory;
+import com.openexchange.oauth.OAuthAPIRegistry;
 import com.openexchange.oauth.OAuthService;
+import com.openexchange.oauth.access.OAuthAccessRegistryService;
+import com.openexchange.oauth.http.OAuthHTTPClientFactory;
 import com.openexchange.oauth.json.AbstractOAuthAJAXActionService;
 import com.openexchange.oauth.json.Services;
 import com.openexchange.oauth.json.oauthaccount.actions.AccountActionFactory;
 import com.openexchange.oauth.json.oauthmeta.actions.MetaDataActionFactory;
 import com.openexchange.oauth.json.proxy.OAuthProxyActionFactory;
+import com.openexchange.oauth.scope.OAuthScopeRegistry;
 import com.openexchange.secret.osgi.tools.WhiteboardSecretService;
 import com.openexchange.session.Session;
 import com.openexchange.tools.session.ServerSession;
@@ -82,7 +86,8 @@ public class OAuthJSONActivator extends AJAXModuleActivator {
 
     @Override
     protected Class<?>[] getNeededServices() {
-        return new Class<?>[] { ConfigurationService.class, DispatcherPrefixService.class, OAuthService.class, OAuthHTTPClientFactory.class, CapabilityService.class };
+        return new Class<?>[] { ConfigurationService.class, DispatcherPrefixService.class, OAuthService.class, OAuthScopeRegistry.class, OAuthHTTPClientFactory.class, CapabilityService.class, ClusterLockService.class,
+            OAuthAccessRegistryService.class, OAuthAPIRegistry.class };
     }
 
     @Override
