@@ -61,6 +61,10 @@ import org.json.JSONObject;
 import com.openexchange.ajax.requesthandler.AJAXRequestData;
 import com.openexchange.exception.OXException;
 import com.openexchange.oauth.API;
+import com.openexchange.oauth.KnownApi;
+import com.openexchange.oauth.json.Services;
+import com.openexchange.server.ServiceExceptionCode;
+import com.openexchange.oauth.OAuthAPIRegistry;
 import com.openexchange.oauth.OAuthAccount;
 import com.openexchange.oauth.OAuthExceptionCodes;
 import com.openexchange.oauth.OAuthService;
@@ -78,14 +82,14 @@ public class OAuthProxyRequest {
 	private String body;
 	private final ServerSession session;
 
-	protected Map<API, List<Pattern>> whitelist = new HashMap<API,List<Pattern>>(){{
-		put(API.LINKEDIN, 	Arrays.asList(Pattern.compile("^http:\\/\\/api\\.linkedin\\.com")));
-		put(API.TWITTER,	Arrays.asList(Pattern.compile("^https?:\\/\\/(.*?\\.)?twitter.com")));
-		put(API.YAHOO, 	Arrays.asList(Pattern.compile("^https?:\\/\\/(.*?\\.)?yahoo(apis)?\\.com")));
-		put(API.TUMBLR, Arrays.asList(Pattern.compile("^https?:\\/\\/.*?\\.tumblr\\.com")));
-		put(API.FLICKR, Arrays.asList(Pattern.compile("^https?:\\/\\/.*?\\.flickr\\.com")));
-		put(API.XING, Arrays.asList(Pattern.compile("^https:\\/\\/api\\.xing\\.com")));
-		put(API.GOOGLE, Arrays.asList(Pattern.compile("^https:\\/\\/www\\.googleapis\\.com")));
+	protected Map<String, List<Pattern>> whitelist = new HashMap<String,List<Pattern>>(){{
+		put(KnownApi.LINKEDIN.getFullName(), 	Arrays.asList(Pattern.compile("^http:\\/\\/api\\.linkedin\\.com")));
+		put(KnownApi.TWITTER.getFullName(),	Arrays.asList(Pattern.compile("^https?:\\/\\/(.*?\\.)?twitter.com")));
+		put(KnownApi.YAHOO.getFullName(), 	Arrays.asList(Pattern.compile("^https?:\\/\\/(.*?\\.)?yahoo(apis)?\\.com")));
+		put(KnownApi.TUMBLR.getFullName(), Arrays.asList(Pattern.compile("^https?:\\/\\/.*?\\.tumblr\\.com")));
+		put(KnownApi.FLICKR.getFullName(), Arrays.asList(Pattern.compile("^https?:\\/\\/.*?\\.flickr\\.com")));
+		put(KnownApi.XING.getFullName(), Arrays.asList(Pattern.compile("^https:\\/\\/api\\.xing\\.com")));
+		put(KnownApi.GOOGLE.getFullName(), Arrays.asList(Pattern.compile("^https:\\/\\/www\\.googleapis\\.com")));
 	}};
 
 	public static enum HTTPMethod {
@@ -164,7 +168,15 @@ public class OAuthProxyRequest {
 		}
 
 		if (req.isSet("api")){
-			API api = API.valueOf(req.getParameter("api").toUpperCase());
+		    OAuthAPIRegistry service = Services.getService(OAuthAPIRegistry.class);
+		    if(service == null){
+		        throw ServiceExceptionCode.absentService(OAuthAPIRegistry.class);
+		    }
+			String sApi = req.getParameter("api");
+            API api = service.resolveFromServiceId(sApi);
+			if (null == api) {
+                throw AjaxExceptionCodes.INVALID_PARAMETER_VALUE.create("api", sApi);
+            }
 
 			return oauthService.getDefaultAccount(api, session);
 		}
@@ -205,7 +217,7 @@ public class OAuthProxyRequest {
 
 	private void whitelist(String checkMe) throws OXException {
 		API proposedApi = getAccount().getAPI();
-		List<Pattern> patterns = whitelist.get(proposedApi);
+		List<Pattern> patterns = whitelist.get(proposedApi.getServiceId());
 		if(patterns == null){
 			throw OAuthExceptionCodes.NOT_A_WHITELISTED_URL.create(checkMe, proposedApi); //TODO: debatable
 		}
